/* assign METADATA library */
libname SGF15 meta library="SGF15";

/* assign PHYSICAL library */
libname sgf15p 'S:\SGF_Paper_Files\2015\data';


options nomprint nosymbolgen nomlogic;
%macro meta_compare(MtableName=, plib=, PtableName=);

   /* debugging - remove when ready */    
   %put Physical Table Name: >>> &PtableName >>>; 
   %put Metadata Table Name: >>> &MtableName >>>; 

   /* check Physical table name is empty. Assign metadata name to physical name macro variable     */
   %if &PtableName = %then %do;
      %let PtableName = &MtableName;
      %put Physical Table Name is now called: >>> &PtableName >>>;
   %end;
 

   /*----------------------------------------------------*/
   /* obtain metadata view of the columns in the dataset */
   /*----------------------------------------------------*/
   %let metaDSN=work.metadata_columns;

   data &metaDSN(keep=column_name format_type column_length);
       length column_name $32.
              column_length $6.
              format_type $200.
              columnURI tableURI metatype $256.
              ;

       /* The CALL MISSING statement initialises the output variables to missing values. */
       call missing (of _all_);

       rc=metadata_resolve("omsobj:PhysicalTable?@Name='&MtableName'",metatype,tableURI);

       /* The METADATA_GETNASN function gets objects associated via the table URI. 
       numcols=metadata_getnasn(tableURI,"Columns",1,columnURI);


       /* The DO statement specifies a group of statements to be executed as a unit.
          The METADATA_GETATTR function gets the values of the Column Name, Column Length and SAS Format attributes. */
       do colnum=1 to numCols;
          numcols=metadata_getnasn(tableURI,"Columns",colnum,columnURI);
          rc=metadata_getattr(columnURI,"SASColumnName",column_name);
          rc=metadata_getattr(columnURI,"SASColumnLength",column_length);
          rc=metadata_getattr(columnURI,"SASFormat",format_type);
          output;
       end;
   run;


   /* convert column names and formats to upcase */
   data &metaDSN;
      set &metaDSN;

      column_name=upcase(column_name);
      format_type=upcase(format_type);
   run;

   /* sort variables into alphabetical order */
   proc sort data=&metaDSN;
      by column_name;
   run;


   /*--------------------------------------*/
   /* export physical columns to a dataset */
   /*--------------------------------------*/
   %let physicalDSN=work.physical_columns;

   proc contents data = &plib..&PtableName noprint                                      
                 out = &physicalDSN(rename=(name=column_name)
                                    keep=name length format formatl formatd
                                    );
   run;

   /* convert column names to uppercase */
   data &physicalDSN(keep= column_name column_length format_type);
      set &physicalDSN;

      column_name=upcase(column_name);
      column_length=put(strip(length),$6.);

      if formatl NE 0 and formatd > 0 then format_type=cats(format,formatl,'.',formatd);
      else if formatl NE 0 then format_type=cats(format,formatl,'.');
      else if formatl = 0 and formatd = 0 and format NE '' then format_type=cats(format,'.');
   run;

   /* sort variables into alphabetical order */
   proc sort data=&physicalDSN;
      by column_name;
   run;

   /*----------------------------------------------*/
   /* compare metadata columns to physical columns */
   /*----------------------------------------------*/
   proc compare base=&physicalDSN
                compare=&metaDSN
                ;
   run;
%mend meta_compare;
%meta_compare(MtableName=sales_analysis, plib=sgf15p, PtableName=)
